const reactTemplates = require('react-templates/src/reactTemplates');
const loaderUtils = require('loader-utils');
const fs = require('fs');
const path = require('path');

function match(content, regex, func = null) {
    const matches = content.match(regex);

    return matches !== null
        ? ( func !== null ? matches.map(func) : matches )
        : null
    ;
}

function aliasPropRt(content) {
    return content.replace(/\@(?=(for|props|repeat|scope|props|class|if)\=(\{|\'|\")(.*?)(\}|\'|\")|stateless)/g, 'rt-');
}

function aliasImportAs(content) {
    return content.replace(/rt-import(.*?)default\=['"](.*?)['"]/g, function(pacakge) {
        return pacakge.replace(/default\=/, 'as=');
    });
}

function aliasImportName(content) {
    return content.replace(/rt-import(.*?)component\=['"](.*?)['"]/g, function(component) {
        return component.replace(/component\=/, 'name=');
    });
}

function setImportName(content) {
    return content.replace(/\<rt\-import\s*(?!name\=)\w+\=['"].*['"]\s*(?!name\=)\w+\=['"].*['"]\s*(?!name\=)\s*\/\>/g, function(tag) {
        return tag.replace('rt-import', 'rt-import name="default"');
    });
}

function defineLoad(content) {
    var i = 0;
    return content.replace(/\<rt\-load.*\/\>/g, function(tag) {
        return tag.replace('rt-load', `rt-import name="default" as="_LOAD_TEMP_VAR_${ i++ }" `);
    });
}

function defineSetDiff(content) {
    return content.replace(/\w+\s*\/\s*\([\w|\,|\s]+\)/g, function(diff) {
        var args = diff.split('/');
        var set  = args[0].trim();
        var subset = args[1].trim()
            .replace(/\(/, '[')
            .replace(/\)/, ']')
            .replace(/[\w|\,|\s]+/, function(s) {
                var atoms = s.split(',');
                var quoted = atoms.map(function(atom) {

                    return `'${atom.replace(/\s*/, '')}'`;
                });

                return quoted.join(',');
            })
        ;

        return `_.omit(${set}, ${subset})`;
    });
}

function defineOnlyIf(content) {
    return content.replace(/\{([^\}]*)if([^\}]*)\}/g, function(match) {
        var args = match.split('if');

        return
            `{${args[1].substr(0, args[1].length - 1).trim()}` +
            `? ${args[0].substr(1).trim()} : null}`
        ;
    });
}

function replaceOnPropQuotes(content) {
    return content.replace(/\son\w*\=["'].+?["']/g, function(on) {
        return on.replace(/[\"|\'].*[\"|\']/, function(quotes) {
            var singleQuotes = quotes.replace(/\"/g, "'");
            return `"{${singleQuotes}}"`;
        });
    });
}

function replaceTagAliases(content) {
    return content
        .replace(/\<import(?=.*\/\>)/g, '<rt-import')
        .replace(/\<load(?=.*\/\>)/g, '<rt-load')
        .replace(/\<scope(?=.*\>)/g, '<rt-scope')
        .replace(/\<\/\s*scope\s*\>/g, '</rt-scope>')
        .replace(/\<template(?=.*\>)/g, '<rt-template')
        .replace(/\<\/\s*template\s*\>/g, '</rt-template>')
        .replace(/\<pre(?=.*\>)/g, '<rt-pre')
        .replace(/\<\/\s*pre\s*\>/g, '</rt-pre>')
        .replace(/\<virtual(?=.*\>)/g, '<rt-virtual')
        .replace(/\<\/\s*virtual\s*\>/g, '</rt-virtual>')
    ;
}

function replacePropAliases(content) {
    return content.replace(/rt\-for/g, 'rt-repeat');
}

function replaceBracesToQuotes(content) {
    return content.replace(/rt\-(?:if|repeat|scope|props)\=\{(.*?)(?!\}\`|\}\}|\}\))\}/g, function(braces) {
        return braces.replace(/\{/, '"').replace(/(?!\}\`|\}\}|\}\))\}/, '"');
    });
}

function replaceQuotesToBraces(content) {
    return content.replace(/\=\{((.|\n)*?)(?!\}\`|\}\}|\}\))\}/g, function(braces) {
        var bracesSingleQuote = braces.replace(/\"/g, "'").replace(/\s+/g, ' ').substr(1);

        return `="${ bracesSingleQuote }"`;
    });
}

function removeLoadImportVars(content) {
    return content.replace(/import\s_LOAD_TEMP_VAR_\d\sfrom/g, 'import');
}

function bindElementProp(content) {
    return content.replace(/\(([^\)]+)\@bind([^\)]+)\)/g, function(block) {
        const nested = block.substr(1).match(/React\.createElement\(([^\)]+)\@bind([^\)]+)\)/);

        if (nested !== null) {
            return block
                .replace(nested[0], bindElementProp(nested[0]))
            ;
        } else {
            const bind = block
                .replace(/\((\w|\.)+\,((\n|.)*?)\@bind['"]\:\s*/, '')
                .replace(/(?!\w|\.)(.|\n)*/, '')
            ;

            return block
                .replace(/\s*['"]\@bind['"]\:\s*(\w|\.)+\,{0,1}/, '')
                .replace(/\((\w|\.)+/, function(el) {
                    return `${ el }.bind(${ bind })`;
                })
            ;
        }
    });
}

function aliasToThisComponents(content) {
    const format = [
        "export default function (props, context) {",
        `\tif(this !== undefined && typeof this.component === 'object') var Component = this.component;`
    ].join('\n');

    return content.replace(/export\s*default\s*function\s*\((.*?)\)\s*\{/, format);
}

function replaceJsCommentsToHtmlComments(content) {
    return content
        .replace(/\{\s*\/\*/g, '<!--')
        .replace(/\*\/\s*\}/g, '-->')
    ;
}

function removeMutlipleLines(content) {
    return content.replace(/\s{2,}/g, " ")
}

module.exports = function(content) {
    var
        parsedContent = null,
        options = loaderUtils.getOptions(this)
    ;

    if (!options.readFileSync) {
        options.readFileSync = fileName => {
            const filePath = path.resolve(path.dirname(this.resourcePath), fileName);
            this.addDependency(filePath);
            return fs.readFileSync(filePath);
        };
    }

    this.cacheable && this.cacheable();

    parsedContent = removeMutlipleLines(content);
    parsedContent = aliasPropRt(parsedContent);
    parsedContent = replaceTagAliases(parsedContent);
    parsedContent = replacePropAliases(parsedContent);
    parsedContent = aliasImportAs(parsedContent);
    parsedContent = aliasImportName(parsedContent);
    parsedContent = setImportName(parsedContent);
    parsedContent = defineLoad(parsedContent);
    parsedContent = defineSetDiff(parsedContent);
    parsedContent = defineOnlyIf(parsedContent);
    parsedContent = replaceOnPropQuotes(parsedContent);
    parsedContent = replaceBracesToQuotes(parsedContent);
    parsedContent = replaceQuotesToBraces(parsedContent);
    parsedContent = replaceJsCommentsToHtmlComments(parsedContent);
    parsedContent = reactTemplates.convertTemplateToReact(parsedContent, options);
    parsedContent = removeLoadImportVars(parsedContent);
    parsedContent = aliasToThisComponents(parsedContent);

    return parsedContent;
};
