/**
 * Helpers
 */

function match(content, regex, func = null) {
    const matches = content.match(regex);

    return matches !== null
        ? ( func !== null ? matches.map(func) : matches )
        : null
    ;
}

function isAutoBind(content) {
    const match = content.match(/\@Autobind/);

    return match !== null;
}

function isExport(content) {
    const match = content.match(/\@export/);

    return match !== null;
}

function isPropsConstructor(content) {
    const match = content.match(/constructor\(props\)/);

    return match !== null;
}

/**
 * Getters
 */

function getClassName(content) {
    const found = match(content, /class\s+\w+/, function(className) {
        return className.replace(/class\s+/, '');
    });

    return (found !== null && found.length > 0) ? found[0] : null;
}

function getStore(content) {
    const match = content.match(/\@Store\s*['"](.*?)['"]/);

    return match !== null ? match[0] : null;
}

function getComponents(content) {
    const match = content.match(/\@Component\s*\{((.|\n)*?)\}/)

    return match !== null ? match[0] : null;
}

function getComponent(content) {
    const components = getComponents(content);

    if (components !== null) {
        return {
            components: true,
            content: components
        };
    } else {
        const match = content.match(/\@Component\s*['"](.*?)['"]/)

        return match === null
            ? {}
            : {
                components: false,
                content: match[0]
            }
        ;
    }
}

/**
 * Parsers
 */

function parseStore(store) {
    return typeof store === 'string'
        ? store.replace(/\@Store\s*['"]/, '').replace(/[\'|\"]/, '')
        : null
    ;
}

function parseComponents(components) {
    if (!!components) {
        const config = [];

        const componentList = components.replace(/(\@Component\s*\{|\})/g, '').split(',');

        componentList.map(function(component) {
            const info = component.split(':');

            if (info.length === 2) {
                config.push({ name: info[0].trim(), path: info[1].trim() });
            }
        });

        return config;
    }

    return [];
}

function parseComponent(component) {
    return component
        .replace(/\@Component\s*['"]/, '')
        .replace(/[\'|\"]/, '')
    ;
}

/**
 * Content Parsers
 */

function commentOutDecorators(content) {
    return content.replace(/(\/\/\s*\@)|\/\*((.|\n)*?)\*\//g, function(decorator) {
        return decorator.replace(/\@/g, '@-');
    });
}

function cleanDecorators(content, decorators) {
    decorators.filter(function(decorator) {
        return decorator !== null;
    }).map(function(decorator) {
        content = content.replace(decorator, '');
    });

    return content.replace(/\@Autobind/, '').trim();
}

function extendAndConstructor(content, component, parsedStore, isAutobind, isProps) {
    content = content.replace(/class\s+\w+\s*\{/, function(match) {
        const clean = match.replace('{', '').trim();

        return `${ clean } extends Component {`;
    });

    const construct = content.match(/constructor\((.*?)\)\s*\{/);
    const bindedComponents = [];

    let renderMethod = 'this.component';

    if (component.components) {
        renderMethod = 'this.component.render';

        parseComponents(component.content).map(function(component) {
            bindedComponents.push(`\t\tthis.component.${ component.name } = LOADER_${ component.name }.bind(this);`);
        });
    } else {
        if (!!component.content) {
            bindedComponents.push('\t\tthis.component = LOADER_component.bind(this);');
        }
    }

    content = construct !== null
        ? content.replace(/constructor\((.*?)\)\s*\{/, function(consrt) {
            return [
                consrt,
                `\t\tsuper(${ isProps ? 'props' : '' });`,
                isAutobind ? '\t\tLOADER_autoBind(this);' : '',
                parsedStore !== null ? '\t\tthis.store = LOADER_store;' : '',
                component.components ? '\t\tthis.component = {};' : '',
                bindedComponents.join('\n'),
            ].join('\n')
        })
        : content.replace(/class(.*?)\{/, function(cls) {
            return [
                cls,
                "\tconstructor() {",
                    `\t\tsuper(${ isProps ? 'props' : '' });`,
                    isAutobind ? '\t\tLOADER_autoBind(this);' : '',
                    parsedStore !== null ? '\t\tthis.store = LOADER_store;' : '',
                    component.components ? '\t\tthis.component = {};' : '',
                    bindedComponents.join('\n'),
                "\t}"
            ].join('\n')
        });

        const renderDefined = content.match(/render\((.*?)\)\s*\{/);
        const render = [
            `\t\tconst Component = ${ renderMethod };`,
            '\t\treturn <Component />;'
        ];

        if (renderDefined === null && !!component.content) {
            content = content.replace(/class(.*?)\{/, function(cls) {
                return [
                    cls,
                    '\trender() {',
                        render.join('\n'),
                    '\t}'
                ].join('\n')
            });
        }

    return content;
}

function importComponent(content, component) {
    let componentImport;

    if (component.components) {
        componentImport = parseComponents(component.content).map(function(v) {
            return `import LOADER_${ v.name } from ${ v.path };`;
        }).join('\n');
    } else {
        if (!!component.content) {
            componentImport = `import LOADER_component from '${ parseComponent(component.content) }';`
        }
    }

    return [
        componentImport,
        content
    ].join('\n');
}

function importStore(content, parsedStore, isAutobind) {
    return [
        isAutobind ? "import LOADER_autoBind from 'react-autobind';" : '',
        parsedStore !== null ? `import LOADER_store from '${parsedStore}';` : '',
        content
    ].join('\n');
}

function importDependencies(content, parsedStore) {
    return [
        "import React, { Component } from 'react';",
        parsedStore !== null ? "import { connectStore } from 'react-trim';" : '',
        content
    ].join('\n')
}

function exportClass(content, className, parsedStore) {
    //const exportDec = isExport(content);

    if (parsedStore !== null) {
        return [
            content,
            `export default connectStore(${className}, {`,
                `\tstore: LOADER_store.redux,`,
                `\tmapStateToConnect: ${className}.mapStateToConnect`,
            `})`
        ].join('\n');
    } else {
        return [
            content,
            `export default ${className}`
        ].join('\n');
    }
}

function staticMapStateToConnect(content, parsedStore) {
    if (parsedStore !== null) {
        const matched = content.match(/mapStateToConnect\((.*?)\)/);

        if (matched !== null) {
            return content.replace(/mapStateToConnect\((.*?)\)/, function(func) {
                return `static ${ func }`;
            });
        } else {
            return content.replace(/class(.*?)\{/, function(cls) {
                return [
                    cls,
                    "\tstatic mapStateToConnect(state) {",
                        "\t\treturn state;",
                    "\t}"
                ].join('\n')
            });
        }
    }

    return content;
}

module.exports = function(content) {
    var className = getClassName(content);

    if (className === null) {
        throw new SyntaxError(
            'ERROR: No Class Defined!\nPlease Define exactly' +
            'one class or use a .js file extension.'
        );
    }

    // Get Meta Data

    var
        parsedContent = commentOutDecorators(content),

        component = getComponent(parsedContent),
        store = getStore(parsedContent),
        isAutobind = isAutoBind(parsedContent),
        isProps = isPropsConstructor(parsedContent),

        parsedStore = parseStore(store)
    ;

    // Begin Parsing

    parsedContent = cleanDecorators(parsedContent, [ component.content, store ]);
    parsedContent = importComponent(parsedContent, component);
    parsedContent = importStore(parsedContent, parsedStore, isAutobind);
    parsedContent = importDependencies(parsedContent, parsedStore);
    parsedContent = staticMapStateToConnect(parsedContent, parsedStore);
    parsedContent = extendAndConstructor(parsedContent, component, parsedStore, isAutobind, isProps);
    parsedContent = exportClass(parsedContent, className, parsedStore);

    // Remove Empty Lines and Return

    return parsedContent.replace(/^\s*\n/gm, '');
};
