/**
 * Helpers
 */

function match(content, regex, func = null) {
    const matches = content.match(regex);

    return matches !== null
        ? ( func !== null ? matches.map(func) : matches )
        : null
    ;
}

function cleanDecorator(decorator) {
    return decorator.replace(/\@\w+\s*/, '');
}

function parser(array) {
    return array.join('\n');
}

/**
 * Getters
 */

function getClassName(content) {
    const found = match(content, /class\s+\w+/, function(className) {
        return className.replace(/class\s+/, '');
    });

    return (found !== null && found.length > 0) ? found[0] : null;
}

function getStores(content) {
    const match = content.match(/\@Stores\s*\{((.|\n)*?)\}/)

    return match !== null ? match[0] : null;
}

function getDecorator(type, content) {
    const metaData = [];
    const regex = new RegExp(`\\@${ type }\\s*\\w+\\((.*?)\\)`, 'g');

    match(content, regex, function(decorator) {
        const cleanedDecorator = cleanDecorator(decorator);
        const functionInfo = cleanedDecorator.replace(/\s|\)/g, '').split('(');
        const name = functionInfo[0];
        const parameters = functionInfo[1].split(',').filter(function(param) {
            return param !== '';
        });

        metaData.push({
            name: name,
            parameters: parameters
        });
    });

    return metaData;
}

/**
 * Content Parsers
 */

function parseStores(stores) {
    if (stores !== null) {
        const config = [];
        const storeList = stores.replace(/(\@Stores\s*\{|\})/g, '').split(',');

        storeList.map(function(store) {
            const info = store.split(':');

            if (info.length === 2) {
                config.push({ name: info[0].trim(), path: info[1].trim() });
            }
        });

        return config;
    }

    return [];
}

function commentOutDecorators(content) {
    return content.replace(/(\/\/\s*\@)|\/\*((.|\n)*?)\*\//g, function(decorator) {
        return decorator.replace(/\@/g, '@-');
    });
}

function cleanDecorators(content, decorators) {
    decorators.filter(function(decorator) {
        return decorator !== null;
    }).map(function(decorator) {
        content = content.replace(decorator, '');
    });

    return content.replace(/\@command\s+/g, '');
}

function importDependencies(content, parsedStores) {
    const imports = parsedStores.map(store => {
        return `import LOADER_${ store.name } from ${ store.path };`;
    });

    return parser([
        "import { Store } from 'react-trim';",
        imports.join('\n'),
        content
    ]);
}

function exportClass(content, className) {
    return parser([
        content,
        `export default new ${ className }();`
    ]);
}

function extendAndConstructor(content, commands, parsedStores) {
    const construct = content.match(/constructor\((.*?)\)\s*\{/);

    const setCommands = commands.map(function(command) {
        return `this.setCommand("${ command.name }", this.${ command.name });`;
    });

    const superParams = parsedStores.map(store => {
        return `\n\t\t\t${ store.name }: LOADER_${ store.name }`;
    });

    content = content.replace(/class\s+\w+\s*\{/, function(match) {
        const clean = match.replace('{', '').trim();
        return `export ${ clean } extends Store {`;
    });

    content = construct === null && (commands.length > 0 || parsedStores.length > 0)
      ? content.replace(/class(.*?)\{/, function(cls) {
          return parser([
            cls,
            "\tconstructor() {",
            '\t\tsuper({',
            superParams.join(','),
            '\t\t});',
            `\t\t${ setCommands.join('\n\t\t') }`,
            "\t}"
          ]);
      })
      : content.replace(/constructor\((.*?)\)\s*\{/, function(consrt) {
          return parser([
            consrt,
            '\t\tsuper({',
            superParams.join(','),
            '\t\t});',
            `\t\t${ setCommands.join('\n\t\t') }`
          ]);
      })
    ;

    return content;
}

module.exports = function(content) {
    var className = getClassName(content);

    if (className === null) {
        throw new SyntaxError(
            'ERROR: No Class Defined!\nPlease Define exactly' +
            'one class or use a .js file extension.'
        );
    }

    // Get Meta Data

    var
        parsedContent = commentOutDecorators(content),
        commands = getDecorator('command', parsedContent),
        stores = getStores(parsedContent),
        parsedStores = parseStores(stores)
    ;

    // Begin Parsing

    parsedContent = cleanDecorators(parsedContent, [ stores ]);
    parsedContent = importDependencies(parsedContent, parsedStores);
    parsedContent = extendAndConstructor(parsedContent, commands, parsedStores);
    parsedContent = exportClass(parsedContent, className);

    // Remove Empty Lines and Return

    return parsedContent.replace(/^\s*\n/gm, '');
}
