const fs = require('fs');
const path = require('path');
const { paths } = require('react-app-rewired');
const ExtractTextPlugin = require("extract-text-webpack-plugin");

const publicPath = paths.servedPath;
const shouldUseRelativeAssetPaths = publicPath === './';
const shouldUseSourceMap = process.env.GENERATE_SOURCEMAP !== 'false';
const cssFilename = 'static/css/[name].[contenthash:8].css';
const extractTextPluginOptions = shouldUseRelativeAssetPaths
    ? { publicPath: Array(cssFilename.split('/').length).join('../') }
    : {};

function resolveApp(dir) {
    return path.resolve(fs.realpathSync(process.cwd()), dir);
}

const resolve = require(resolveApp('resolve.js'));

module.exports = function override(config, env) {
    /**
     * Resolve Folder
     *
     * Instead of:
     * import '../../containers/something'
     *
     * You can resolve it to:
     * import 'containers/something'
     */
    if(typeof resolve.modules === 'function') {
        config.resolve.modules = resolve.modules(resolveApp);
    } else {
        config.resolve.modules = [
            resolveApp('src'),
            resolveApp('node_modules'),
        ];
    }

    /**
     * Resolve Extensions
     *
     * Instead of:
     * import 'containers/something.jsc'
     *
     * You can resolve it to:
     * import 'containers/something'
     */
    if(typeof resolve.extensions === 'function') {
        config.resolve.extensions.unshift(...resolve.extensions());
    } else {
        config.resolve.extensions.unshift(...[
            '.jsc',
            '.jss',
            '.jscp',
            '.mcss',
            '.mscss',
            '.css',
            '.scss',
        ]);
    }

    if (env === "production") {
        config.module.rules[1].oneOf.splice(2, 0, ...[
            // Container Loader
            {
                test: /\.jsc$/,
                include: paths.appSrc,
                use: [
                    {
                        loader: require.resolve('babel-loader'),
                        options: {
                            compact: true,
                            presets: require.resolve("babel-preset-react-app"),
                        },
                    },
                    require.resolve('react-trim/loader/container'),
                ],
            },
            // Store Loader
            {
                test: /\.jss$/,
                include: paths.appSrc,
                use: [
                    {
                        loader: require.resolve('babel-loader'),
                        options: {
                            compact: true,
                            presets: require.resolve("babel-preset-react-app"),
                        },
                    },
                    require.resolve('react-trim/loader/store'),
                ],
            },
            // Component Loader
            {
                test: /\.jscp$/,
                include: paths.appSrc,
                use: [
                    {
                        loader: require.resolve('babel-loader'),
                        options: {
                            compact: true,
                            presets: require.resolve("babel-preset-react-app"),
                        },
                    },
                    {
                        loader: require.resolve('react-trim/loader/component'),
                        options: { modules: "es6" },
                    },
                ],
            },
            // Modular CSS Loader
            {
                test: /\.mcss$/,
                loader: ExtractTextPlugin.extract(Object.assign({
                    fallback: {
                        loader: require.resolve('style-loader'),
                        options: { hmr: false },
                    },
                    use: [
                        {
                            loader: require.resolve('css-loader'),
                            options: {
                                importLoaders: 1,
                                modules: true,
                                localIdentName: "x-[hash:base64:5]",
                                minimize: true,
                                sourceMap: shouldUseSourceMap,
                            },
                        },
                        {
                            loader: require.resolve('postcss-loader'),
                            options: {
                                ident: 'postcss',
                                plugins: () => [
                                    require('postcss-flexbugs-fixes'),
                                    autoprefixer({
                                        browsers: [
                                        '>1%',
                                        'last 4 versions',
                                        'Firefox ESR',
                                        'not ie < 9',
                                        ],
                                        flexbox: 'no-2009',
                                    }),
                                ],
                            },
                        },
                    ],
                }, extractTextPluginOptions)),
            },
            // CSS Loader
            {
                test: /\.css$/,
                loader: ExtractTextPlugin.extract(Object.assign(
                {
                    fallback: {
                        loader: require.resolve('style-loader'),
                        options: { hmr: false },
                    },
                    use: [
                        {
                            loader: require.resolve('css-loader'),
                            options: {
                                importLoaders: 1,
                                minimize: true,
                                sourceMap: shouldUseSourceMap,
                            },
                        },
                        {
                            loader: require.resolve('postcss-loader'),
                            options: {
                                ident: 'postcss',
                                plugins: () => [
                                require('postcss-flexbugs-fixes'),
                                    autoprefixer({
                                        browsers: [
                                        '>1%',
                                        'last 4 versions',
                                        'Firefox ESR',
                                        'not ie < 9',
                                        ],
                                        flexbox: 'no-2009',
                                    }),
                                ],
                            },
                        },
                    ],
                }, extractTextPluginOptions)),
            },
            // Modular SCSS Loader
            {
                test: /\.mscss$/,
                loader: ExtractTextPlugin.extract(Object.assign({
                        fallback: {
                            loader: require.resolve('style-loader'),
                            options: {
                                hmr: false,
                            },
                        },
                        use: [
                            {
                                loader: require.resolve('css-loader'),
                                options: {
                                    importLoaders: 1,
                                    modules: true,
                                    localIdentName: "x-[hash:base64:5]",
                                    minimize: true,
                                    sourceMap: shouldUseSourceMap,
                                },
                            },
                            {
                                loader: require.resolve('sass-loader'),
                                options: {
                                    includePaths: [ paths.appNodeModules ],
                                },
                            },
                        ],
                    }, extractTextPluginOptions)
                ),
            },
            // SCSS Loader
            {
                test: /\.scss$/,
                loader: ExtractTextPlugin.extract(Object.assign({
                        fallback: {
                            loader: require.resolve('style-loader'),
                            options: {
                                hmr: false,
                            },
                        },
                        use: [
                            {
                                loader: require.resolve('css-loader'),
                                options: {
                                    importLoaders: 1,
                                    minimize: true,
                                    sourceMap: shouldUseSourceMap,
                                },
                            },
                            {
                                loader: require.resolve('sass-loader'),
                                options: {
                                    includePaths: [ paths.appNodeModules ],
                                },
                            },
                        ],
                    }, extractTextPluginOptions),
                ),
            },
        ]);

        return config;
    } else {
        config.module.rules[1].oneOf.splice(2, 0, ...[
            // Container Loader
            {
                test: /\.jsc$/,
                include: paths.appSrc,
                use: [
                    {
                        loader: require.resolve("babel-loader"),
                        options: {
                            babelrc: false,
                            cacheDirectory: true,
                            presets: require.resolve("babel-preset-react-app"),
                        },
                    },
                    require.resolve("react-trim/loader/container"),
                ],
            },
            // Store Loader
            {
                test: /\.jss$/,
                include: paths.appSrc,
                use: [
                    {
                        loader: require.resolve("babel-loader"),
                        options: {
                            babelrc: false,
                            cacheDirectory: true,
                            presets: require.resolve("babel-preset-react-app"),
                        },
                    },
                    require.resolve("react-trim/loader/store"),
                ],
            },
            // Component Loader
            {
                test: /\.jscp$/,
                include: paths.appSrc,
                use: [
                    {
                        loader: require.resolve("babel-loader"),
                        options: { cacheDirectory: true },
                    },
                    {
                        loader: require.resolve("react-trim/loader/component"),
                        options: { modules: "es6" },
                    },
                ],
            },
            // Modular CSS Loader
            {
                test: /\.mcss$/,
                use: [
                    require.resolve("style-loader"),
                    {
                        loader: require.resolve("css-loader"),
                        options: {
                            importLoaders: 1,
                            modules: true,
                            localIdentName: "[local]-[hash:base64:5]",
                        },
                    },
                    {
                        loader: require.resolve("postcss-loader"),
                        options: {
                            ident: "postcss",
                            plugins: () => [
                                require("postcss-flexbugs-fixes"),
                                autoprefixer({
                                    browsers: [
                                        ">1%",
                                        "last 4 versions",
                                        "Firefox ESR",
                                        "not ie < 9",
                                    ],
                                    flexbox: "no-2009",
                                }),
                            ],
                        },
                    },
                ],
            },
            // CSS Loader
            {
                test: /\.css$/,
                use: [
                    require.resolve('style-loader'),
                    {
                        loader: require.resolve('css-loader'),
                        options: { importLoaders: 1 },
                    },
                    {
                        loader: require.resolve('postcss-loader'),
                        options: {
                            ident: 'postcss',
                            plugins: () => [
                                require('postcss-flexbugs-fixes'),
                                autoprefixer({
                                    browsers: [
                                        '>1%',
                                        'last 4 versions',
                                        'Firefox ESR',
                                        'not ie < 9',
                                    ],
                                    flexbox: 'no-2009',
                                }),
                            ],
                        },
                    },
                ],
            },
            // Modular SCSS Loader
            {
                test: /\.mscss$/,
                use: [
                    require.resolve("style-loader"),
                    {
                        loader: require.resolve("css-loader"),
                        options: {
                            importLoaders: 1,
                            modules: true,
                            localIdentName: "[local]-[hash:base64:5]",
                        },
                    },
                    {
                        loader: require.resolve("sass-loader"),
                        options: { includePaths: [paths.appNodeModules] },
                    },
                ],
            },
            // SCSS Loader
            {
                test: /\.scss$/,
                use: [
                    require.resolve("style-loader"),
                    {
                        loader: require.resolve("css-loader"),
                        options: { importLoaders: 1 },
                    },
                    {
                        loader: require.resolve("sass-loader"),
                        options: { includePaths: [paths.appNodeModules] },
                    },
                ],
            },
        ]);

        return config;
    }

}
